
////////////////////////////////////////////////////////////////////// 
// 
// MVCannyEdge.cpp: implementation of CMVCannyEdge class.
// 
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "MVCannyEdge.h"
#define _USE_MATH_DEFINES
#include <math.h>

BYTE * Gausscopy;	//temporary storage for the gauss filtered image and others

short * SobelMag;	//temp storage for sobel operated image
BYTE * SobelRad;

double sqr2;		//some constants for the gauss filter
double sqr15;
double divis;

double thigh;		//parameters for the canny filter.
double tlow;

int dx[8] = {1,1,0,-1,-1,-1,0,1};	//some global stepping directions
int dy[8] = {0,1,1,1,0,-1,-1,-1};

HRESULT CMVCannyEdge::InitVars(void) 
{

	thigh=0.9;		//play with these for changes to the hysteresis loop of edge filtering
	tlow=0.5;


	sqr2=sqrt(2.0);	//populate the constants
	sqr15=sqrt(15.0);
	divis=2*sqr2+6+sqr15;
	gSrcSize=min(gSrcSize,yImage*nRow);
	Gausscopy=(BYTE*)malloc(gSrcSize);	//allocate and clear the storage space
	SobelRad=(BYTE*)malloc(gSrcSize);
	SobelMag=(short*)malloc(gSrcSize*sizeof(short));
	ZeroMemory(SobelRad,gSrcSize);
	ZeroMemory(SobelMag,gSrcSize*sizeof(short));
	CopyFrame=false;
	
	FrameCnt=0;
	return S_OK;
}

BYTE CMVCannyEdge::GetGaussianV(BYTE* pin)
//this function returns the v filtered 1.4sigma gaussian. 
{
	return ((*(pin-2*nRow)*sqr2+
				*(pin-1*nRow)*3+
				*(pin)*sqr15+
				*(pin+1*nRow)*3+
				*(pin+2*nRow)*sqr2)/divis);
}
BYTE CMVCannyEdge::GetGaussianH(BYTE* pin)
{
//this function returns the h filtered 1.4sigma gaussian. 
	return ((*(pin-2*nPixel)*sqr2+
				*(pin-1*nPixel)*3+
				*(pin)*nPixel+
				*(pin+1*nPixel)*3+
				*(pin+2*nPixel)*sqr2)/divis);
}

int CMVCannyEdge::xOperator(BYTE* pin)
//this function returns the h filtered sobel operator
{
	return (*(pin+nRow-nPixel)*-1
				+*(pin-nPixel)*-2
				+*(pin-nRow-nPixel)*-1
				+*(pin+nRow+nPixel)*1
				+*(pin+nPixel)*2
				+*(pin-nRow+nPixel)*1);
}
int CMVCannyEdge::yOperator(BYTE* pin)
//this function returns the v filtered sobel operator
{
	return (*(pin+nRow-nPixel)*1
				+*(pin+nRow)*2
				+*(pin+nRow+nPixel)*1
				+*(pin-nRow+nPixel)*-1
				+*(pin-nRow)*-2
				+*(pin-nRow-nPixel)*-1);
}

HRESULT CMVCannyEdge::TransformFrame(BYTE * pInData, BYTE * pOutData) 
{

	//Gaussian filter
	for (int x=2;x<xImage-2;x++)		//loop around the columns
	{
		for (int y=2;y<yImage-2;y++)	//process every row in the current column
		{									
			*(Gausscopy+y*nRow+x*nPixel)=GetGaussianV(pInData+y*nRow+x*nPixel);
			*(Gausscopy+y*nRow+x*nPixel+1)=GetGaussianV(pInData+y*nRow+x*nPixel+1);
			*(Gausscopy+y*nRow+x*nPixel+2)=GetGaussianV(pInData+y*nRow+x*nPixel+2);
		}
	}
	for (int x=2;x<xImage-2;x++)		//loop around the columns
	{
		for (int y=2;y<yImage-2;y++)	//process every row in the current column
		{						
			*(pOutData+y*nRow+x*nPixel)=GetGaussianH(Gausscopy+y*nRow+x*nPixel);
			*(pOutData+y*nRow+x*nPixel+1)=GetGaussianH(Gausscopy+y*nRow+x*nPixel+1);
			*(pOutData+y*nRow+x*nPixel+2)=GetGaussianH(Gausscopy+y*nRow+x*nPixel+2);
		}
	}
	
// Now Sobel operators
	for (int x=2;x<xImage-2;x++)		//loop around the columns
	{
		for (int y=2;y<yImage-2;y++)	//process every row in the current column
		{	
			double dx,dy,angle;
			dx=xOperator(pOutData+y*nRow+x*nPixel+1);
			dy=yOperator(pOutData+y*nRow+x*nPixel+1);
			*(SobelMag+y*nRow+x*nPixel+1)=abs(dx)+abs(dy);	// note that the mag will always be even.

			angle=M_PI+atan2(dy,dx);

			//descretise the angles into 4 bins
			if (angle<M_PI/8)
				*(SobelRad+y*nRow+x*nPixel+1)=0;
			else if (angle<3*M_PI/8)
				*(SobelRad+y*nRow+x*nPixel+1)=80;
			else if (angle<5*M_PI/8)
				*(SobelRad+y*nRow+x*nPixel+1)=160;
			else if (angle<7*M_PI/8)
				*(SobelRad+y*nRow+x*nPixel+1)=255;
			else if (angle<9*M_PI/8)
				*(SobelRad+y*nRow+x*nPixel+1)=0;
			else if (angle<11*M_PI/8)
				*(SobelRad+y*nRow+x*nPixel+1)=80;
			else if (angle<13*M_PI/8)
				*(SobelRad+y*nRow+x*nPixel+1)=160;
			else if (angle<15*M_PI/8)
				*(SobelRad+y*nRow+x*nPixel+1)=255;
			else
				*(SobelRad+y*nRow+x*nPixel+1)=0;
		}
	}

//remove non-maximal edge components and 
//build the histogram of edge responses
	int  hist[2050];
	ZeroMemory(hist,2050*sizeof(int));
	int totp=0;
	for (int x=2;x<xImage-2;x++)		//loop around the columns
	{
		for (int y=2;y<yImage-2;y++)	//process every row in the current column
		{	
			if (*(SobelRad+y*nRow+x*nPixel+1)==0)	//vertical component, so check l/r for higher
			{
				if (*(SobelMag+y*nRow+x*nPixel+1)<*(SobelMag+(y)*nRow+(x+1)*nPixel+1)||
					*(SobelMag+y*nRow+x*nPixel+1)<*(SobelMag+(y)*nRow+(x-1)*nPixel+1))
				{
					*(SobelMag+y*nRow+x*nPixel+1)=0;
				}
				else
				{
					hist[*(SobelMag+y*nRow+x*nPixel+1)]++;
					totp++;
				}
			}
			else if (*(SobelRad+y*nRow+x*nPixel+1)==80) //topleft to bottomright diag component, so check ur/dl
			{
				if (*(SobelMag+y*nRow+x*nPixel+1)<*(SobelMag+(y-1)*nRow+(x-1)*nPixel+1)||
					*(SobelMag+y*nRow+x*nPixel+1)<*(SobelMag+(y+1)*nRow+(x+1)*nPixel+1))
				{
					*(SobelMag+y*nRow+x*nPixel+1)=0;
				}
				else
				{
					hist[*(SobelMag+y*nRow+x*nPixel+1)]++;
					totp++;
				}
			}
			else if (*(SobelRad+y*nRow+x*nPixel+1)==160) //horizontal component, so check u/d
			{
				if (*(SobelMag+y*nRow+x*nPixel+1)<*(SobelMag+(y+1)*nRow+(x)*nPixel+1)||
					*(SobelMag+y*nRow+x*nPixel+1)<*(SobelMag+(y-1)*nRow+(x)*nPixel+1))
				{
					*(SobelMag+y*nRow+x*nPixel+1)=0;
				}
				else
				{
					hist[*(SobelMag+y*nRow+x*nPixel+1)]++;
					totp++;
				}
			}
			else if (*(SobelRad+y*nRow+x*nPixel+1)==255)//topright to bottomleft diagonal component, so check dr/ul
			{
				if (*(SobelMag+y*nRow+x*nPixel+1)<*(SobelMag+(y-1)*nRow+(x+1)*nPixel+1)||
					*(SobelMag+y*nRow+x*nPixel+1)<*(SobelMag+(y+1)*nRow+(x-1)*nPixel+1))
				{
					*(SobelMag+y*nRow+x*nPixel+1)=0;
				}
				else
				{
					hist[*(SobelMag+y*nRow+x*nPixel+1)]++;
					totp++;
				}
			}

		}
	}

	//now get threshholds
	int highcount= (int)(totp*thigh+0.5);
	

int idx=0;
int acc=0;
	while(idx<2050)
	{
		acc+=hist[idx];
		if (acc>highcount)break;
		idx+=2;		//note that Sobel operator |x|+|y| must be even, ie multiples of 2!
	}
	highthresh=idx;
	lowthresh=int(highthresh*tlow+0.5);
	
	ZeroMemory(Gausscopy,gSrcSize);


//now perform hysteresis edge checking
// ie every edge sequence > lowthresh that is connected to an edge > highthresh is ok, all others removed
	for (int x=2;x<xImage-2;x++)		//loop around the columns
	{
		for (int y=2;y<yImage-2;y++)	//process every row in the current column
		{	
			if(*(SobelMag+y*nRow+x*nPixel+1)>highthresh)
			{
				*(SobelMag+y*nRow+x*nPixel+1)=-1;
				FillMemory(Gausscopy+y*nRow+x*nPixel,3,0xff);
				CheckEdges(x,y);	//iterative process to get all connected chains
			}
			else
			{
				*(SobelMag+y*nRow+x*nPixel+1)=0;
			}
		}
	}
	CopyMemory(pOutData,Gausscopy,gSrcSize);
	Overlay(pOutData,"Canny Edge Detector");
	return S_OK;
}
void CMVCannyEdge::CheckEdges(int x,int y)
{
//iteritive subroutine to check all connected chains
   for(int i=0;i<8;i++)
   {

	   if (*(SobelMag+(x+dx[i])*nPixel+(y+dy[i])*nRow+1)>lowthresh)
	   {
		   *(SobelMag+(x+dx[i])*nPixel+(y+dy[i])*nRow+1)=-1;
		   FillMemory(Gausscopy+(x+dx[i])*nPixel+(y+dy[i])*nRow,3,0xff);
			CheckEdges((x+dx[i]),(y+dy[i]));
	   }
	   else
	   {
		   if ( *(SobelMag+(x+dx[i])*nPixel+(y+dy[i])*nRow+1)!=-1)
			FillMemory(Gausscopy+(x+dx[i])*nPixel+(y+dy[i])*nRow,3,0);
	   }
        
         
     }
   

}
void CMVCannyEdge::SetPix(BYTE * pDestData,int x,int y,int r,int g,int b)
{
	BYTE * pBGR=pDestData+y*nRow+x*nPixel;


	*pBGR=b;			//blue channel is first
	*(pBGR+1)=g;		//then green
	*(pBGR+2)=r;		//then red
}
short CMVCannyEdge::GetB(BYTE * pSrc,int x,int y)
{
	return *(pSrc+y*nRow+x*nPixel);			//blue channel is first
}
short CMVCannyEdge::GetR(BYTE * pSrc,int x,int y)
{
	return *(pSrc+y*nRow+x*nPixel+2);		//red channel is +2
}
short CMVCannyEdge::GetG(BYTE * pSrc,int x,int y)
{
	return *(pSrc+y*nRow+x*nPixel+1);		//green channel is +1
}
///////////////////////////////////////////////////////////////////////
// CMVCannyEdge: Constructor
///////////////////////////////////////////////////////////////////////
CMVCannyEdge::CMVCannyEdge(TCHAR *tszName, LPUNKNOWN punk, HRESULT *phr)
    :	CvtFilter(tszName,punk, CLSID_MVCannyEdge)
{
}

///////////////////////////////////////////////////////////////////////
// ~CMVCannyEdge: Destructor
///////////////////////////////////////////////////////////////////////
CMVCannyEdge::~CMVCannyEdge()
{
	
	//free any globals you malloc'd
}

  



