
////////////////////////////////////////////////////////////////////// 
// 
// MVHoughCircle.cpp: implementation of CMVHoughCircle class.
// 
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "MVHoughCircle.h"
#define _USE_MATH_DEFINES
#include <math.h>

/* This filter implements the Hough technique for circle detection. The relevant
	parameters are stepsize,ParameterThresh and EdgeThresh

	*/

HRESULT CMVHoughCircle::InitVars(void) 
{

	CopyFrame=true;
	//set CopyFrame to true for OutData to be initialised with a copy of InData
	//set false otherwise
	
	FrameCnt=0;
	return S_OK;
}
int CMVHoughCircle::xOperator(BYTE* pin)
//this function returns the h filtered sobel operator
{
	return (*(pin+nRow-nPixel)*-1
				+*(pin-nPixel)*-2
				+*(pin-nRow-nPixel)*-1
				+*(pin+nRow+nPixel)*1
				+*(pin+nPixel)*2
				+*(pin-nRow+nPixel)*1);
}
int CMVHoughCircle::yOperator(BYTE* pin)
//this function returns the v filtered sobel operator
{
	return (*(pin+nRow-nPixel)*1
				+*(pin+nRow)*2
				+*(pin+nRow+nPixel)*1
				+*(pin-nRow+nPixel)*-1
				+*(pin-nRow)*-2
				+*(pin-nRow-nPixel)*-1);
}

HRESULT CMVHoughCircle::TransformFrame(BYTE * pInData, BYTE * pOutData) 
{
	//do your process here

	double angle,dx,dy;
	int stepsize=1;				//change this to change the radius of the circles detected
	int ParameterThresh=120;	//change this to change how much of a circle is to be found
	int EdgeThresh=50;			//change this to change contrast of edges


	//initialise the parameter space
	for (int i=0;i<10;i++)
	{		
		ZeroMemory(Hough[i],80000*sizeof(int));
	}
	
	//search for edges
	for (int x=1;x<xImage-1;x+=1)		//loop around the columns
	{
		for (int y=1;y<yImage-1;y+=1)	//process every row in the current column
		{
			//find an edge
			dx=xOperator(pInData+y*nRow+x*nPixel+1);
			dy=yOperator(pInData+y*nRow+x*nPixel+1);
			angle=M_PI/2+atan2(-dy,dx);
			if (abs(dx>EdgeThresh)||abs(dy)>EdgeThresh)
			{
				double gx,gy;
				double ddx=sin(angle);
				double ddy=cos(angle);
				
				//add a possible circle center at every point normal to the found edge
				for (int idx=0;idx<10;idx++)
				{					
					(Hough[idx][min(max(0,int(y+(idx*stepsize+10)*ddy)),yImage)*xImage+min(max(0,int(x+(10+idx*stepsize)*ddx)),xImage)])+=10;
					(Hough[idx][min(max(0,int(y-(idx*stepsize+10)*ddy)),yImage)*xImage+min(max(0,int(x-(10+idx*stepsize)*ddx)),xImage)])+=10;
				}

			}
		}
	}

	//now search parameter space for peaks
	for (int x=1;x<xImage-1;x++)		//loop around the columns
	{
		for (int y=1;y<yImage-1;y+=1)	//process every row in the current column
		{
			for (int i=0;i<10;i++)
			{
				if((Hough[i][y*xImage+x])>ParameterThresh)
				{

					*(pOutData+y*nRow+x*nPixel)+=50;
					DrawCircle(x,y,i*stepsize+10,pOutData);
				}

			}
		}
	}

	Overlay(pOutData,"Hough Circle Detection");
	return S_OK;
}
void CMVHoughCircle::DrawCircle(int a,int b,int r, BYTE * pOutData)
{
	for (double i=0;i<2*M_PI;i+=0.01)
	{
		FillMemory((pOutData+min(max(0,int(a+r*sin(i))),xImage)*nPixel+min(max(0,int(b+r*cos(i))),yImage)*nRow),3,0);
	}
}
void CMVHoughCircle::SetPix(BYTE * pDestData,int x,int y,int r,int g,int b)
{
	BYTE * pBGR=pDestData+y*nRow+x*nPixel;


	*pBGR=b;			//blue channel is first
	*(pBGR+1)=g;		//then green
	*(pBGR+2)=r;		//then red
}
short CMVHoughCircle::GetB(BYTE * pSrc,int x,int y)
{
	return *(pSrc+y*nRow+x*nPixel);			//blue channel is first
}
short CMVHoughCircle::GetR(BYTE * pSrc,int x,int y)
{
	return *(pSrc+y*nRow+x*nPixel+2);		//red channel is +2
}
short CMVHoughCircle::GetG(BYTE * pSrc,int x,int y)
{
	return *(pSrc+y*nRow+x*nPixel+1);		//green channel is +1
}
///////////////////////////////////////////////////////////////////////
// CMVHoughCircle: Constructor
///////////////////////////////////////////////////////////////////////
CMVHoughCircle::CMVHoughCircle(TCHAR *tszName, LPUNKNOWN punk, HRESULT *phr)
    :	CvtFilter(tszName,punk, CLSID_MVHoughCircle)
{
}

///////////////////////////////////////////////////////////////////////
// ~CMVHoughCircle: Destructor
///////////////////////////////////////////////////////////////////////
CMVHoughCircle::~CMVHoughCircle()
{
	//free any globals you malloc'd
}

  



