// MVHoughCircle.h: interface for the CMVHoughCircle class.
// 
//////////////////////////////////////////////////////////////////////
//  CMVHoughCircle: (A DirectShow filter)
//
//  Purpose: This filter is a basic Transform Filter to process video
//        
//  Usage: 
//  1.  Register the filter. 
//      regsvr32 MVHoughCircle.ax
//
//  2.  Insert the filter into the graph to process video data (live 
//      capture or video files)
//      a.  Use the GraphEdt.exe (in DirectX SDK) to build a graph 
//      b.  To use this filter in App, include iMVHoughCircle.h for
//          definition of interfaces and build the graph in the App.
///////////////////////////////////////////////////////////////////////
#pragma once



#include <streams.h>     // DirectShow
#include <initguid.h>    // DEFINE_GUID to declare an EXTERN_C const.
#include <stdio.h>
#include <tchar.h>
#include <vtdxf.h>


DEFINE_GUID(CLSID_MVHoughCircle,0x25b34b87, 0x7259, 0x41bc, 0xaa, 0xe3, 0x9f, 0x6c, 0x1, 0x6b, 0xc1, 0x24);
DEFINE_GUID(IID_IMVHoughCircle, 0xe64dc546, 0x18d5, 0x433c, 0x8c, 0xc, 0x2, 0x69, 0x7c, 0x5d, 0x39, 0x10);


class CMVHoughCircle:
	public CvtFilter
{
public:

	    DECLARE_IUNKNOWN;


    static CUnknown *WINAPI CreateInstance(LPUNKNOWN punk, HRESULT *phr);

	CMVHoughCircle::CMVHoughCircle(TCHAR *tszName, LPUNKNOWN punk, HRESULT *phr);
	~CMVHoughCircle();
	
	 HRESULT InitVars(void) ;
	 HRESULT TransformFrame(BYTE * pInData, BYTE * pOutData) ;
	 HRESULT Overlay(BYTE *pDestData,LPSTR Str);
	void SetPix(BYTE *,int x,int y,int r,int g, int b);
	short GetB(BYTE *,int x,int y);
	short GetG(BYTE *,int x,int y);
	short GetR(BYTE *,int x,int y);
    HBITMAP     m_DibSection;
    HDC         m_DC;
    void *      m_pDibBits;
	 
	
	int Hough[10][80000];
	void DrawCircle(int a,int b,int r, BYTE * pOutData);
	//put any local subs or variables here
	int xOperator(BYTE* pin);
	int yOperator(BYTE* pin);
	

};

	
/////////////////////
// DllRegisterServer
/////////////////////
STDAPI DllRegisterServer()
{
    return AMovieDllRegisterServer2( TRUE );
}

////////////////////////
// DllUnregisterServer
////////////////////////
STDAPI DllUnregisterServer()
{
    return AMovieDllRegisterServer2( FALSE );
}


AM_MEDIA_TYPE *pMediaType;
VIDEOINFOHEADER *pvi;

#define TRANSFORM_NAME L"MVHoughCircle Filter"

const AMOVIESETUP_FILTER sudMVHoughCircle =
{ &CLSID_MVHoughCircle              // clsID
, TRANSFORM_NAME                    // strName
, MERIT_DO_NOT_USE                  // dwMerit
, 0                                 // nPins
, NULL};                       // lpPin

// Needed for the CreateInstance mechanism
CFactoryTemplate g_Templates[]=
    {   
		{ TRANSFORM_NAME
        , &CLSID_MVHoughCircle
        , CMVHoughCircle::CreateInstance
        , NULL
        , &sudMVHoughCircle
		}
    };
int g_cTemplates = sizeof(g_Templates)/sizeof(g_Templates[0]);




///////////////////////////////////////////////////////////////////////
// CreateInstance: for COM to create a MVHoughCircle object
///////////////////////////////////////////////////////////////////////
CUnknown * WINAPI CMVHoughCircle::CreateInstance(LPUNKNOWN punk, HRESULT *phr) {

    CMVHoughCircle *pNewObject = new CMVHoughCircle(NAME("MVHoughCircle"), punk, phr );
    if (pNewObject == NULL) {
        *phr = E_OUTOFMEMORY;
    }

    return pNewObject;
}


HRESULT CMVHoughCircle::Overlay(BYTE *pDestData,LPSTR Str)
{

    if( FrameCnt<=1)
    {
		VIDEOINFO* pVI = (VIDEOINFO*) m_pInput->CurrentMediaType( ).Format( );
          m_DibSection = CreateDIBSection(
                                        NULL,
                                        (BITMAPINFO*) &pVI->bmiHeader,
                                        DIB_RGB_COLORS,
                                        &m_pDibBits,
                                        NULL,
                                        0
                                        );
        if( !m_DibSection )
        {
            return E_FAIL;
        }

        HDC hdc = GetDC( NULL );
        m_DC = CreateCompatibleDC( hdc );
        if( !m_DC )
            return E_FAIL;

        ReleaseDC( NULL, hdc );

         SelectObject( m_DC, m_DibSection );
        
		SetTextColor(m_DC, RGB(255,0,0));

		HFONT hf;
	    static LOGFONT lf;  // structure for font information  
 
    // Get a handle to the ANSI fixed-pitch font, and copy 
    // information about the font to a LOGFONT structure. 
 
		GetObject(GetStockObject(ANSI_FIXED_FONT), sizeof(LOGFONT), 
			&lf); 
 
        lf.lfWeight = 700; 
		lf.lfItalic = 0;
		lf.lfUnderline=0;
		lf.lfHeight = -MulDiv(48, GetDeviceCaps(m_DC, LOGPIXELSY), 72);
 
    // Create the font, and then return its handle.  
 
		hf=  CreateFont(lf.lfHeight,0, 
        lf.lfEscapement, lf.lfOrientation, lf.lfWeight, 
        lf.lfItalic, lf.lfUnderline, lf.lfStrikeOut, lf.lfCharSet, 
        lf.lfOutPrecision, lf.lfClipPrecision, lf.lfQuality, 
        lf.lfPitchAndFamily, lf.lfFaceName); 
		
		SetBkMode (m_DC, TRANSPARENT);
		SetTextAlign(m_DC,TA_CENTER);
		
    }
    
    memcpy( m_pDibBits, pDestData,gSrcSize );    

    // Draw the current frame
    TCHAR szText[256];
    wsprintf( szText, TEXT("%i %s"),FrameCnt,Str);
    BOOL bWorked = TextOut(m_DC, 100, 10, szText, _tcslen( szText ) );

    // Copy our modified bitmap into the original sample buffer
    memcpy( pDestData, m_pDibBits, gSrcSize );

return NOERROR;
}
