// MVSobelEdge.cpp: implementation of CMVSobelEdge class.
// 
//////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////// 
// 
// MVSobelEdge.cpp: implementation of CMVSobelEdge class.
// 
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "MVSobelEdge.h"
#define _USE_MATH_DEFINES
#include <math.h>
/* This filter implements the sobel edge detection technique. The relevant
	parameters are GxThresh and GyThresh

	The sobel operator can be defined as a combination of the convolution masks:
				1	2	1					-1	0	1
		Gy=		0	0	0			Gx=		-2	0	2
				-1	-2	-1					-1	0	1

	Edge Detection then becomes a simple threshhold.			 
*/


HRESULT CMVSobelEdge::InitVars(void) 
{

	//play with these values
	GxThresh=100;
	GyThresh=100;	

	CopyFrame=false;
	//set CopyFrame to true for OutData to be initialised with a copy of InData
	//set false otherwise
		
	FrameCnt=0;
	return S_OK;
}
int CMVSobelEdge::xOperator(BYTE* pin)
{
	int size=1;
	return (*(pin+size*nRow-size*nPixel)*-1
				+*(pin-size*nPixel)*-2
				+*(pin-size*nRow-size*nPixel)*-1
				+*(pin+size*nRow+size*nPixel)*1
				+*(pin+size*nPixel)*2
				+*(pin-size*nRow+size*nPixel)*1);
}
int CMVSobelEdge::yOperator(BYTE* pin)
{
	int size=1;
	return (*(pin+size*nRow-size*nPixel)*1
				+*(pin+size*nRow)*2
				+*(pin+size*nRow+size*nPixel)*1
				+*(pin-size*nRow+size*nPixel)*-1
				+*(pin-size*nRow)*-2
				+*(pin-size*nRow-size*nPixel)*-1);
}
HRESULT CMVSobelEdge::TransformFrame(BYTE * pInData, BYTE * pOutData) 
{
	
	//do your process here

	for (int x=10;x<xImage-10;x+=1)		//loop around the columns
	{
		for (int y=10;y<yImage-10;y+=1)	//process every row in the current column
		{
			int offset=1;	//green channel
			double dx,dy,angleb,angleg,angler;
			double magg,magr,magb;

			dx=xOperator(pInData+y*nRow+x*nPixel+offset);
			dy=yOperator(pInData+y*nRow+x*nPixel+offset);

			magg=abs(dx)+abs(dy);	// note that the mag will always be even. 
									// it would be more precise to use sqrt(dx*dx+dy*dy), 
									// but we are only interested in a threshhold
			
			angleg=M_PI+atan2(dy,dx);	//this has the direction of the edge - useful for finding
										// particular lines

			if(dx>GxThresh||dy>GyThresh)
				FillMemory(pOutData+y*nRow+x*nPixel,3,0xff);
			else
				FillMemory(pOutData+y*nRow+x*nPixel,3,0x00);
		}
	}
	char msg[100];
	sprintf(msg,"Sobel edge detection, thresh: %i/%i",GxThresh,GyThresh);
	Overlay(pOutData,msg);
	return S_OK;
}

void CMVSobelEdge::SetPix(BYTE * pDestData,int x,int y,int r,int g,int b)
{
	BYTE * pBGR=pDestData+y*nRow+x*nPixel;


	*pBGR=b;			//blue channel is first
	*(pBGR+1)=g;		//then green
	*(pBGR+2)=r;		//then red
}
short CMVSobelEdge::GetB(BYTE * pSrc,int x,int y)
{
	return *(pSrc+y*nRow+x*nPixel);			//blue channel is first
}
short CMVSobelEdge::GetR(BYTE * pSrc,int x,int y)
{
	return *(pSrc+y*nRow+x*nPixel+2);		//red channel is +2
}
short CMVSobelEdge::GetG(BYTE * pSrc,int x,int y)
{
	return *(pSrc+y*nRow+x*nPixel+1);		//green channel is +1
}
///////////////////////////////////////////////////////////////////////
// CMVSobelEdge: Constructor
///////////////////////////////////////////////////////////////////////
CMVSobelEdge::CMVSobelEdge(TCHAR *tszName, LPUNKNOWN punk, HRESULT *phr)
    :	CvtFilter(tszName,punk, CLSID_MVSobelEdge)
{
}

///////////////////////////////////////////////////////////////////////
// ~CMVSobelEdge: Detructor
///////////////////////////////////////////////////////////////////////
CMVSobelEdge::~CMVSobelEdge()
{
	
	//free any globals you malloc'd
}

  



