// MVSobelEdge.h: interface for the CMVSobelEdge class.
// 
//////////////////////////////////////////////////////////////////////
//  CMVSobelEdge: (A DirectShow filter)
//
//  Purpose: This filter is a basic Transform Filter to process video
//        
//  Usage: 
//  1.  Register the filter. 
//      regsvr32 MVSobelEdge.ax
//
//  2.  Insert the filter into the graph to process video data (live 
//      capture or video files)
//      a.  Use the GraphEdt.exe (in DirectX SDK) to build a graph 
//      b.  To use this filter in App, include iMVSobelEdge.h for
//          definition of interfaces and build the graph in the App.
///////////////////////////////////////////////////////////////////////
#pragma once



#include <streams.h>     // DirectShow
#include <initguid.h>    // DEFINE_GUID to declare an EXTERN_C const.
#include <stdio.h>
#include <tchar.h>
#include <vtdxf.h>


DEFINE_GUID(CLSID_MVSobelEdge,0xb2586b0, 0x971c, 0x44ea, 0x89, 0x71, 0x71, 0x90, 0x7c, 0x71, 0xe9, 0x42);
DEFINE_GUID(IID_IMVSobelEdge, 0x3242ebe6, 0xe50f, 0x45bc, 0x94, 0x8c, 0x15, 0x36, 0xcc, 0x14, 0x3, 0xfb);


class CMVSobelEdge:
	public CvtFilter
{
public:

	    DECLARE_IUNKNOWN;


    static CUnknown *WINAPI CreateInstance(LPUNKNOWN punk, HRESULT *phr);

	CMVSobelEdge::CMVSobelEdge(TCHAR *tszName, LPUNKNOWN punk, HRESULT *phr);
	~CMVSobelEdge();
	
	 HRESULT InitVars(void) ;
	 HRESULT TransformFrame(BYTE * pInData, BYTE * pOutData) ;
	 HRESULT Overlay(BYTE *pDestData,LPSTR Str);
	void SetPix(BYTE *,int x,int y,int r,int g, int b);
	short GetB(BYTE *,int x,int y);
	short GetG(BYTE *,int x,int y);
	short GetR(BYTE *,int x,int y);
	int xOperator(BYTE* pin);
	int yOperator(BYTE* pin);
    HBITMAP     m_DibSection;
    HDC         m_DC;
    void *      m_pDibBits;
	 

	
	
	//put any local subs or variables here
	int GxThresh;
	int GyThresh;

};

	
/////////////////////
// DllRegisterServer
/////////////////////
STDAPI DllRegisterServer()
{
    return AMovieDllRegisterServer2( TRUE );
}

////////////////////////
// DllUnregisterServer
////////////////////////
STDAPI DllUnregisterServer()
{
    return AMovieDllRegisterServer2( FALSE );
}


AM_MEDIA_TYPE *pMediaType;
VIDEOINFOHEADER *pvi;

#define TRANSFORM_NAME L"MVSobelEdge Filter"

const AMOVIESETUP_FILTER sudMVSobelEdge =
{ &CLSID_MVSobelEdge              // clsID
, TRANSFORM_NAME                    // strName
, MERIT_DO_NOT_USE                  // dwMerit
, 0                                 // nPins
, NULL};                       // lpPin

// Needed for the CreateInstance mechanism
CFactoryTemplate g_Templates[]=
    {   
		{ TRANSFORM_NAME
        , &CLSID_MVSobelEdge
        , CMVSobelEdge::CreateInstance
        , NULL
        , &sudMVSobelEdge
		}
    };
int g_cTemplates = sizeof(g_Templates)/sizeof(g_Templates[0]);




///////////////////////////////////////////////////////////////////////
// CreateInstance: for COM to create a MVSobelEdge object
///////////////////////////////////////////////////////////////////////
CUnknown * WINAPI CMVSobelEdge::CreateInstance(LPUNKNOWN punk, HRESULT *phr) {

    CMVSobelEdge *pNewObject = new CMVSobelEdge(NAME("MVSobelEdge"), punk, phr );
    if (pNewObject == NULL) {
        *phr = E_OUTOFMEMORY;
    }

    return pNewObject;
}


HRESULT CMVSobelEdge::Overlay(BYTE *pDestData,LPSTR Str)
{

    if( FrameCnt<=1)
    {
		VIDEOINFO* pVI = (VIDEOINFO*) m_pInput->CurrentMediaType( ).Format( );
          m_DibSection = CreateDIBSection(
                                        NULL,
                                        (BITMAPINFO*) &pVI->bmiHeader,
                                        DIB_RGB_COLORS,
                                        &m_pDibBits,
                                        NULL,
                                        0
                                        );
        if( !m_DibSection )
        {
            return E_FAIL;
        }

        HDC hdc = GetDC( NULL );
        m_DC = CreateCompatibleDC( hdc );
        if( !m_DC )
            return E_FAIL;

        ReleaseDC( NULL, hdc );

         SelectObject( m_DC, m_DibSection );
        
		SetTextColor(m_DC, RGB(255,0,0));

		HFONT hf;
	    static LOGFONT lf;  // structure for font information  
 
    // Get a handle to the ANSI fixed-pitch font, and copy 
    // information about the font to a LOGFONT structure. 
 
		GetObject(GetStockObject(ANSI_FIXED_FONT), sizeof(LOGFONT), 
			&lf); 
 
        lf.lfWeight = 700; 
		lf.lfItalic = 0;
		lf.lfUnderline=0;
		lf.lfHeight = -MulDiv(48, GetDeviceCaps(m_DC, LOGPIXELSY), 72);
 
    // Create the font, and then return its handle.  
 
		hf=  CreateFont(lf.lfHeight,0, 
        lf.lfEscapement, lf.lfOrientation, lf.lfWeight, 
        lf.lfItalic, lf.lfUnderline, lf.lfStrikeOut, lf.lfCharSet, 
        lf.lfOutPrecision, lf.lfClipPrecision, lf.lfQuality, 
        lf.lfPitchAndFamily, lf.lfFaceName); 
		
		SetBkMode (m_DC, TRANSPARENT);
		SetTextAlign(m_DC,TA_CENTER);
		
    }
    
    memcpy( m_pDibBits, pDestData,gSrcSize );    

    // Draw the current frame
    TCHAR szText[256];
    wsprintf( szText, TEXT("%i %s"),FrameCnt,Str);
    BOOL bWorked = TextOut(m_DC, 100, 10, szText, _tcslen( szText ) );

    // Copy our modified bitmap into the original sample buffer
    memcpy( pDestData, m_pDibBits, gSrcSize );

return NOERROR;
}
