// vtdxf.h - common defines for Mark Dunn Base filter

class CvtFilter;

// ==================================================
// Implements the input pin
// ==================================================

class CvtFilterInputPin : public CBaseInputPin
{
    friend class CvtFilter;

protected:
    CvtFilter *m_pvtFilter;


public:

    CvtFilterInputPin(
        TCHAR *pObjectName,
        CvtFilter *pvtFilter,
        HRESULT * phr,
        LPCWSTR pName);
#ifdef UNICODE
    CvtFilterInputPin(
        char *pObjectName,
        CvtFilter *pvtFilter,
        HRESULT * phr,
        LPCWSTR pName);
#endif

    STDMETHODIMP QueryId(LPWSTR * Id)
    {
        return AMGetWideString(L"Input", Id);
    }

    // Grab and release extra interfaces if required

    HRESULT CheckConnect(IPin *pPin);
    HRESULT BreakConnect();
    HRESULT CompleteConnect(IPin *pReceivePin);

    // check that we can support this output type
    HRESULT CheckMediaType(const CMediaType* mtIn);

    // set the connection media type
    HRESULT SetMediaType(const CMediaType* mt);

    // --- IMemInputPin -----

    // here's the next block of data from the stream.
    // AddRef it yourself if you need to hold it beyond the end
    // of this call.
    STDMETHODIMP Receive(IMediaSample * pSample);

    // provide EndOfStream that passes straight downstream
    // (there is no queued data)
    STDMETHODIMP EndOfStream(void);

    // passes it to CvtFilter::BeginFlush
    STDMETHODIMP BeginFlush(void);

    // passes it to CvtFilter::EndFlush
    STDMETHODIMP EndFlush(void);

    STDMETHODIMP NewSegment(
                        REFERENCE_TIME tStart,
                        REFERENCE_TIME tStop,
                        double dRate);

    // Check if it's OK to process samples
    virtual HRESULT CheckStreaming();

    // Media type
public:
    CMediaType& CurrentMediaType() { return m_mt; };

};

// ==================================================
// Implements the output pin
// ==================================================

class CvtFilterOutputPin : public CBaseOutputPin
{
    friend class CvtFilter;

protected:
    CvtFilter *m_pvtFilter;

public:

    // implement IMediaPosition by passing upstream
    IUnknown * m_pPosition;

    CvtFilterOutputPin(
        TCHAR *pObjectName,
        CvtFilter *pvtFilter,
        HRESULT * phr,
        LPCWSTR pName);
#ifdef UNICODE
    CvtFilterOutputPin(
        CHAR *pObjectName,
        CvtFilter *pvtFilter,
        HRESULT * phr,
        LPCWSTR pName);
#endif
    ~CvtFilterOutputPin();

    // override to expose IMediaPosition
    STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void **ppv);

    // --- CBaseOutputPin ------------

    STDMETHODIMP QueryId(LPWSTR * Id)
    {
        return AMGetWideString(L"Output", Id);
    }

    // Grab and release extra interfaces if required

    HRESULT CheckConnect(IPin *pPin);
    HRESULT BreakConnect();
    HRESULT CompleteConnect(IPin *pReceivePin);

    // check that we can support this output type
    HRESULT CheckMediaType(const CMediaType* mtOut);

    // set the connection media type
    HRESULT SetMediaType(const CMediaType *pmt);

    // called from CBaseOutputPin during connection to ask for
    // the count and size of buffers we need.
    HRESULT DecideBufferSize(
                IMemAllocator * pAlloc,
                ALLOCATOR_PROPERTIES *pProp);

    // returns the preferred formats for a pin
    HRESULT GetMediaType(int iPosition,CMediaType *pMediaType);

    // inherited from IQualityControl via CBasePin
    STDMETHODIMP Notify(IBaseFilter * pSender, Quality q);

    // Media type
public:
    CMediaType& CurrentMediaType() { return m_mt; };
};


class AM_NOVTABLE CvtFilter : public CBaseFilter
{

public:

    // map getpin/getpincount for base enum of pins to owner
    // override this to return more specialised pin objects

    virtual int GetPinCount();
    virtual CBasePin * GetPin(int n);
    STDMETHODIMP FindPin(LPCWSTR Id, IPin **ppPin);

    // override state changes to allow derived vtFilter filter
    // to control streaming start/stop
    STDMETHODIMP Stop();
    STDMETHODIMP Pause();

public:

    CvtFilter(TCHAR *, LPUNKNOWN, REFCLSID clsid);
#ifdef UNICODE
    CvtFilter(CHAR *, LPUNKNOWN, REFCLSID clsid);
#endif
    ~CvtFilter();

	int gSrcSize;
	int  nPixel;
    int xImage   ;
    int yImage    ;
	int nRow ;
	long FrameCnt;
	
	bool CopyFrame;

	HRESULT CheckTransform(const CMediaType *mtIn, const CMediaType *mtOut);
	BOOL canPerformTransform(const CMediaType *pMediaType);
    // These can be supplied in a derived class

    HRESULT Transform(IMediaSample * pIn, IMediaSample *pOut);

    // check if you can support mtIn
    HRESULT CheckInputType(const CMediaType* mtIn);

    // check if you can support the vtFilter from this input to this output
    //HRESULT CheckTransform(const CMediaType* mtIn, const CMediaType* mtOut) ;

    // this goes in the factory template table to create new instances
    // static CCOMObject * CreateInstance(LPUNKNOWN, HRESULT *);

    // call the SetProperties function with appropriate arguments
     HRESULT DecideBufferSize(
                        IMemAllocator * pAllocator,
                        ALLOCATOR_PROPERTIES *pprop) ;

    // override to suggest OUTPUT pin media types
    HRESULT GetMediaType(int iPosition, CMediaType *pMediaType) ;
HRESULT copyMediaSample(IMediaSample *pSource, IMediaSample *pDest) const;
	virtual HRESULT InitVars(void) PURE;
	virtual HRESULT TransformFrame(BYTE * pInData, BYTE * pOutData) PURE;

    // =================================================================
    // ----- Optional Override Methods           -----------------------
    // =================================================================

    // you can also override these if you want to know about streaming
    virtual HRESULT StartStreaming();
    virtual HRESULT StopStreaming();

    // override if you can do anything constructive with quality notifications
    virtual HRESULT AlterQuality(Quality q);

    // override this to know when the media type is actually set
    virtual HRESULT SetMediaType(PIN_DIRECTION direction,const CMediaType *pmt);

    // chance to grab extra interfaces on connection
    virtual HRESULT CheckConnect(PIN_DIRECTION dir,IPin *pPin);
    virtual HRESULT BreakConnect(PIN_DIRECTION dir);
    virtual HRESULT CompleteConnect(PIN_DIRECTION direction,IPin *pReceivePin);

    // chance to customize the vtFilter process
    virtual HRESULT Receive(IMediaSample *pSample);

    // Standard setup for output sample
    HRESULT InitializeOutputSample(IMediaSample *pSample, IMediaSample **ppOutSample);

    // if you override Receive, you may need to override these three too
    virtual HRESULT EndOfStream(void);
    virtual HRESULT BeginFlush(void);
    virtual HRESULT EndFlush(void);
    virtual HRESULT NewSegment(
                        REFERENCE_TIME tStart,
                        REFERENCE_TIME tStop,
                        double dRate);

#ifdef PERF
    // Override to register performance measurement with a less generic string
    // You should do this to avoid confusion with other filters
    virtual void RegisterPerfId()
         {m_idvtFilter = MSR_REGISTER(TEXT("vtFilter"));}
#endif // PERF


// implementation details

public:

#ifdef PERF
    int m_idvtFilter;                 // performance measuring id
#endif
    BOOL m_bEOSDelivered;              // have we sent EndOfStream
    BOOL m_bSampleSkipped;             // Did we just skip a frame
    BOOL m_bQualityChanged;            // Have we degraded?

    // critical section protecting filter state.

    CCritSec m_csFilter;

    // critical section stopping state changes (ie Stop) while we're
    // processing a sample.
    //
    // This critical section is held when processing
    // events that occur on the receive thread - Receive() and EndOfStream().
    //
    // If you want to hold both m_csReceive and m_csFilter then grab
    // m_csFilter FIRST - like CvtFilter::Stop() does.

    CCritSec m_csReceive;

    // these hold our input and output pins

    friend class CvtFilterInputPin;
    friend class CvtFilterOutputPin;
    CvtFilterInputPin *m_pInput;
    CvtFilterOutputPin *m_pOutput;
};
